package uk.ac.gla.photofight;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.media.SoundPool;
import android.util.Log;

public class Fighter {
	
	public static final int MAX_LIFE = 100;
	
	public String name;
	public Bitmap walkBitmap;
	public Bitmap standBitmap;
	public Bitmap punchBitmap;
	public Bitmap kickBitmap;
	public Bitmap takeHitBitmap;
	public Bitmap crouchBitmap;
	public Bitmap victoryBitmap;
	public Bitmap defeatBitmap;

	public int hitVoiceResource;
	public int takeHitVoiceResource;
	public int winningVoiceResource;
	public int victoryVoiceResource;
	public int defeatVoiceResource;
	
	public double x;
	public double y;
	public double dx;
	public double dy;
	
//	public boolean isJumping;
//	public boolean isWalking;
	
	public Bitmap currentBitmap;
    public long currentBitmapLastUpdateTime;
    
    public int health;
	
	public Fighter(Context context, String name) {
		this.name = name;
		
		Log.i("Fighter constructor", "started");

		BitmapFactory.Options options = new BitmapFactory.Options();
		options.inSampleSize = 8;
		
		walkBitmap = BitmapFactory.decodeFile(Storage.getOutputMediaFile(context, name, "walk.png").getAbsolutePath(), options);
		Log.i("Fighter constructor", "loaded walk.png");
		standBitmap = BitmapFactory.decodeFile(Storage.getOutputMediaFile(context, name, "stand.png").getAbsolutePath(), options);
		currentBitmap = standBitmap;
		Log.i("Fighter constructor", "loaded stand.png");
		punchBitmap = BitmapFactory.decodeFile(Storage.getOutputMediaFile(context, name, "punch.png").getAbsolutePath(), options);
		Log.i("Fighter constructor", "loaded punch.png");
		kickBitmap = BitmapFactory.decodeFile(Storage.getOutputMediaFile(context, name, "kick.png").getAbsolutePath(), options);
		takeHitBitmap = BitmapFactory.decodeFile(Storage.getOutputMediaFile(context, name, "take_hit.png").getAbsolutePath(), options);
		crouchBitmap = BitmapFactory.decodeFile(Storage.getOutputMediaFile(context, name, "crouch.png").getAbsolutePath(), options);
		victoryBitmap = BitmapFactory.decodeFile(Storage.getOutputMediaFile(context, name, "victory.png").getAbsolutePath(), options);
		defeatBitmap = BitmapFactory.decodeFile(Storage.getOutputMediaFile(context, name, "defeat.png").getAbsolutePath(), options);
		
		x = 0;
		y = 0;
		dx = 0;
		dy = 0;
//		isJumping = false;
		
		currentBitmapLastUpdateTime = 0;
		health = MAX_LIFE;
	}
	
	public Fighter(Context context, String name, boolean mirrored) {
		this(context, name);
		
		if (mirrored) {
//			Log.i("Fighter constructor", "mirrored ON");
//			Matrix matrix = new Matrix();
//			matrix.preScale(-1, 1);
//
//			walkBitmap = Bitmap.createBitmap(walkBitmap, 0, 0, walkBitmap.getWidth(), walkBitmap.getHeight(), matrix, false);
//			standBitmap = Bitmap.createBitmap(standBitmap, 0, 0, standBitmap.getWidth(), standBitmap.getHeight(), matrix, false);
//			punchBitmap = Bitmap.createBitmap(punchBitmap, 0, 0, punchBitmap.getWidth(), punchBitmap.getHeight(), matrix, false);
//			kickBitmap = Bitmap.createBitmap(kickBitmap, 0, 0, kickBitmap.getWidth(), kickBitmap.getHeight(), matrix, false);
//			takeHitBitmap = Bitmap.createBitmap(takeHitBitmap, 0, 0, takeHitBitmap.getWidth(), takeHitBitmap.getHeight(), matrix, false);
//			crouchBitmap = Bitmap.createBitmap(crouchBitmap, 0, 0, crouchBitmap.getWidth(), crouchBitmap.getHeight(), matrix, false);
//			victoryBitmap = Bitmap.createBitmap(victoryBitmap, 0, 0, victoryBitmap.getWidth(), victoryBitmap.getHeight(), matrix, false);
//			defeatBitmap = Bitmap.createBitmap(defeatBitmap, 0, 0, defeatBitmap.getWidth(), defeatBitmap.getHeight(), matrix, false);
//			currentBitmap = standBitmap;
			mirrorBitmaps();
		}
		Log.i("Fighter constructor", "finished");
	}
	
	public void mirrorBitmaps() {
		Matrix matrix = new Matrix();
		matrix.preScale(-1, 1);

		walkBitmap = Bitmap.createBitmap(walkBitmap, 0, 0, walkBitmap.getWidth(), walkBitmap.getHeight(), matrix, false);
		standBitmap = Bitmap.createBitmap(standBitmap, 0, 0, standBitmap.getWidth(), standBitmap.getHeight(), matrix, false);
		punchBitmap = Bitmap.createBitmap(punchBitmap, 0, 0, punchBitmap.getWidth(), punchBitmap.getHeight(), matrix, false);
		kickBitmap = Bitmap.createBitmap(kickBitmap, 0, 0, kickBitmap.getWidth(), kickBitmap.getHeight(), matrix, false);
		takeHitBitmap = Bitmap.createBitmap(takeHitBitmap, 0, 0, takeHitBitmap.getWidth(), takeHitBitmap.getHeight(), matrix, false);
		crouchBitmap = Bitmap.createBitmap(crouchBitmap, 0, 0, crouchBitmap.getWidth(), crouchBitmap.getHeight(), matrix, false);
		victoryBitmap = Bitmap.createBitmap(victoryBitmap, 0, 0, victoryBitmap.getWidth(), victoryBitmap.getHeight(), matrix, false);
		defeatBitmap = Bitmap.createBitmap(defeatBitmap, 0, 0, defeatBitmap.getWidth(), defeatBitmap.getHeight(), matrix, false);
		currentBitmap = standBitmap;
	}
	
	public void loadVoiceIntegers(Context context, SoundPool sp) {
		hitVoiceResource = sp.load(Storage.getOutputMediaFile(context, name, "hit.3gpp").getAbsolutePath(), 1);
		takeHitVoiceResource = sp.load(Storage.getOutputMediaFile(context, name, "take_hit.3gpp").getAbsolutePath(), 1);
		winningVoiceResource = sp.load(Storage.getOutputMediaFile(context, name, "winning.3gpp").getAbsolutePath(), 1);
		victoryVoiceResource = sp.load(Storage.getOutputMediaFile(context, name, "victory.3gpp").getAbsolutePath(), 1);
		defeatVoiceResource = sp.load(Storage.getOutputMediaFile(context, name, "defeat.3gpp").getAbsolutePath(), 1);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Fighter other = (Fighter) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

}
