package uk.ac.gla.photofight;

import java.util.List;

import android.app.Activity;
import android.content.Context;
import android.graphics.Typeface;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.TextView;

public class FighterArrayAdapter extends ArrayAdapter<Fighter> {
	private final Activity context;
	private final List<Fighter> fighterList;
	private int resourceId;
	private Typeface tf;

	/**
	 * Constructor
	 * 
	 * @param context
	 *            the application content
	 * @param resourceId
	 *            the ID of the resource/view
	 * @param fighterList
	 *            the bound List
	 */
	public FighterArrayAdapter(Activity context, int resourceId,
			List<Fighter> fighterList) {
		super(context, resourceId, fighterList);
		this.context = context;
		this.resourceId = resourceId;
		this.fighterList = fighterList;

        tf = Typeface.createFromAsset(context.getAssets(), "hot-mustard-btn.ttf");
	}

	/**
	 * Updates the view
	 * 
	 * @param position
	 *            the ArrayList position to update
	 * @param convertView
	 *            the view to update/inflate if needed
	 * @param parent
	 *            the groups parent view
	 */
	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		LayoutInflater inflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		View row = inflater.inflate(resourceId, parent, false);

		Fighter f = fighterList.get(position);
		TextView rowTxt = (TextView) row
				.findViewById(R.id.fighter_row_textView);
		rowTxt.setText(f.name);
		rowTxt.setTypeface(tf);
		ImageView rowImg = (ImageView) row
				.findViewById(R.id.fighter_row_imageView);
		rowImg.setImageBitmap(f.victoryBitmap);

		return row;
	}

}
