package uk.ac.gla.photofight;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import uk.ac.gla.photofight.personalize.InsertMediaToCharacterActivity;

import android.content.Context;
import android.util.Log;
import android.widget.Toast;

public class Storage {

	public static boolean isSdPresent() {
		return android.os.Environment.getExternalStorageState().equals(
				android.os.Environment.MEDIA_MOUNTED);
	}

	/** Create a File for saving an image */
	public static File getOutputMediaFile(Context context, String fighterName,
			String filename) {
		// To be safe, you should check that the SDCard is mounted
		// using Environment.getExternalStorageState() before doing this.
		if (!isSdPresent()) {
			Toast.makeText(context, "You should first mount an SD card",
					Toast.LENGTH_SHORT).show();
			return null;
		}

		File mediaStorageDir = new File(context.getExternalFilesDir(null)
				.getAbsolutePath()
				+ File.separator
				+ "Players"
				+ File.separator + fighterName);

		// Create the storage directory if it does not exist
		if (!mediaStorageDir.exists()) {
			if (!mediaStorageDir.mkdirs()) {
				Log.d("PhotoFighter", "failed to create directory");
				return null;
			}
		}

		// Create a media file name
		File mediaFile;
		mediaFile = new File(mediaStorageDir.getPath() + File.separator
				+ File.separator + filename);

		return mediaFile;
	}

	public static List<String> getFighterNames(Context context) {
		List<String> result = new ArrayList<String>();
		File mediaStorageDir = new File(context.getExternalFilesDir(null) + File.separator +
				"Players");
		File[] playerDirectories = mediaStorageDir.listFiles();
		if (playerDirectories == null) {
			Log.i("getPlayerNames()", "Invalid directory");
		} else {
			for (File f : playerDirectories) {
				Log.i("File/Directory found in Players/", f.getName());
				if (f.isDirectory() && isComplete(context, f.getName())) {
					result.add(f.getName());
					Log.i("Player added to list", f.getName());
				}
			}
		}
		return result;
	}
	
	public static void deleteFighter(Context context, String fighterName) {
		File mediaStorageDir = new File(context.getExternalFilesDir(null)
				.getAbsolutePath()
				+ File.separator
				+ "Players"
				+ File.separator + fighterName);
		deleteRecursive(mediaStorageDir);
	}
	
	private static void deleteRecursive(File fileOrDirectory) {
	    if (fileOrDirectory.isDirectory())
	        for (File child : fileOrDirectory.listFiles())
	            deleteRecursive(child);

	    fileOrDirectory.delete();
	}
	
	public static boolean isComplete(Context context, String fighterName) {
		return getOutputMediaFile(context, fighterName, "walk.png").exists() &&
			getOutputMediaFile(context, fighterName, "stand.png").exists() &&
			getOutputMediaFile(context, fighterName, "punch.png").exists() &&
			getOutputMediaFile(context, fighterName, "kick.png").exists() &&
			getOutputMediaFile(context, fighterName, "take_hit.png").exists() &&
			getOutputMediaFile(context, fighterName, "crouch.png").exists() &&
			getOutputMediaFile(context, fighterName, "victory.png").exists() &&
			getOutputMediaFile(context, fighterName, "defeat.png").exists() &&
			getOutputMediaFile(context, fighterName, "hit.3gpp").exists() &&
			getOutputMediaFile(context, fighterName, "take_hit.3gpp").exists() &&
			getOutputMediaFile(context, fighterName, "winning.3gpp").exists() &&
			getOutputMediaFile(context, fighterName, "victory.3gpp").exists() &&
			getOutputMediaFile(context, fighterName, "defeat.3gpp").exists();
	}


}
