package uk.ac.gla.photofight.fight;

import java.util.Random;

import uk.ac.gla.photofight.Fighter;
import uk.ac.gla.photofight.fight.FightView.FightThread;
import android.util.Log;

public class ControlFighterThread extends Thread {

	private boolean mRun;
	// The fighter controlled by the thread
	private Fighter mainFighter;
	// His opponent
	private Fighter opponent;
	
	private FightThread fightThread;
	
	private long updatePlanTime;
	private long now;
	
	private Plan plan;
	private Random r;
	
	private enum Plan {
		DEFENSIVE,
		OFFENSIVE,
		NEUTRAL,
		CRAZY
	}
	
	public ControlFighterThread(FightThread fightThread, Fighter mainFighter, Fighter opponent) {
		this.fightThread = fightThread;
		this.mainFighter = mainFighter;
		this.opponent = opponent;
		
		updatePlanTime = System.currentTimeMillis();

		r = new Random();
		selectNewPlan();
		
	}
	
	public void run() {
		while (mRun && !fightThread.fightFinished) {
			now = System.currentTimeMillis();
			if (now - updatePlanTime > 5000) {
				selectNewPlan();
				updatePlanTime = now;
			}
			try {
				followPlan();
			} catch (InterruptedException e) {	}
		}
		Log.i("ControlFighterThread", "finished");
	}

	public void setRunning(boolean b) {
		mRun = b;
	}
	
	private void selectNewPlan() {
		if (mainFighter.health < 0.25)
			plan = Plan.DEFENSIVE;
		else {
			int i = r.nextInt(4);
			plan = Plan.values()[i];
		}
//		plan = Plan.OFFENSIVE;
		Log.i("ControlFighter", plan.toString());
	}
	
	private void followPlan() throws InterruptedException {
		double distance = mainFighter.x - opponent.x; 
		switch (plan) {
		case DEFENSIVE:
			if (Math.abs(distance) < 100) {
				if (mainFighter.x  + (FightThread.FIGHTER_SIZE_X / 2) < fightThread.mCanvasWidth &&
							mainFighter.x - (FightThread.FIGHTER_SIZE_X / 2) > 0 ) {
					fightThread.walk(mainFighter, (int) (mainFighter.x - opponent.x));
					sleep(200);
					fightThread.stopWalk(mainFighter);
				} else {
					if (r.nextBoolean())
						fightThread.jump(mainFighter);
					else { 
						fightThread.crouch(mainFighter);
						sleep(1000);
						fightThread.stopCrouch(mainFighter);
					}				
				}
			} else 
				walkCasually();
			break;
		case OFFENSIVE:
			if (Math.abs(distance) >= FightThread.PROXIMITY_X) {
				fightThread.walk(mainFighter, (int) (-mainFighter.x + opponent.x));
				sleep(200);
				fightThread.stopWalk(mainFighter);
				sleep(100);
			} else {
//				if(r.nextBoolean())
//					fightThread.punch(mainFighter);
//				else
					fightThread.kick(mainFighter);
				sleep(500);
			}				
			break;
		case NEUTRAL:
			switch (r.nextInt(3)) {
			case 0: 
				walkCasually();
				break;
			case 1:
				fightThread.crouch(mainFighter);
				sleep(1000);
				fightThread.stopCrouch(mainFighter);
				break;
			case 2:
				fightThread.jump(mainFighter);	
				break;
			}
			break;
		case CRAZY:
			switch (r.nextInt(5)) {
			case 0: 
				walkCasually();
				break;
			case 1:
				fightThread.crouch(mainFighter);
				sleep(1000);
				fightThread.stopCrouch(mainFighter);
				break;
			case 2:
				fightThread.jump(mainFighter);	
				break;
			case 3:
				fightThread.punch(mainFighter);
				break;
			case 4:
				fightThread.kick(mainFighter);
				break;
			}
			break;
		}
	}

	private void walkCasually() throws InterruptedException {
		if (r.nextBoolean()) {
			if (r.nextBoolean())
				fightThread.walk(mainFighter, 1);
			else
				fightThread.walk(mainFighter, -1);
			sleep(200);
			fightThread.stopWalk(mainFighter);
		} else
			sleep(200);
	}

}
