package uk.ac.gla.photofight.fight;

import uk.ac.gla.photofight.Fighter;
import uk.ac.gla.photofight.R;
import android.content.Context;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.RectF;
import android.graphics.Typeface;
import android.media.AudioManager;
import android.media.SoundPool;
import android.media.SoundPool.OnLoadCompleteListener;
import android.util.AttributeSet;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class FightView extends SurfaceView implements SurfaceHolder.Callback {
	
	private Context context;

	private final int Y_REFERENCE = 250;
	private final int MY_FIGHTER_STARTING_X = 200;
	private final int OPPONENT_STARTING_X = 600;

	// private Context mContext;
	private FightThread thread;

	private Fighter myFighter;
	private Fighter opponent;
	private Bitmap backgroundBitmap;

	// private Rect backgroundRect;

	public FightView(Context context, AttributeSet attrs) {
		super(context, attrs);

		this.context = context;
		SurfaceHolder holder = getHolder();
		holder.addCallback(FightView.this);

		SharedPreferences settings = context.getSharedPreferences("photoFight",
				0);
		myFighter = new Fighter(context, settings.getString("myFighterName",
				null), false);
		myFighter.x = MY_FIGHTER_STARTING_X;
		myFighter.y = Y_REFERENCE;
		opponent = new Fighter(context,
				settings.getString("opponentName", null), true);
		opponent.x = OPPONENT_STARTING_X;
		opponent.y = Y_REFERENCE;

		backgroundBitmap = BitmapFactory.decodeResource(context.getResources(),
				R.drawable.dojo);

		thread = new FightThread(holder);
	}
	
	public FightThread getFightThread() {
		return thread;
	}

	public void rightCursorDown() {
		Log.i("Cursor Movement", "Right down");
		thread.walk(myFighter, 1);
	}

	public void rightCursorUp() {
		thread.stopWalk(myFighter);
	}

	public void leftCursorDown() {
		thread.walk(myFighter, -1);
	}

	public void leftCursorUp() {
		thread.stopWalk(myFighter);
	}

	public void downCursorDown() {
		thread.crouch(myFighter);
	}

	public void downCursorUp() {
		thread.stopCrouch(myFighter);
	}

	public void upClicked() {
		thread.jump(myFighter);
	}

	public void kickClicked() {
		thread.kick(myFighter);
	}

	public void punchClicked() {
		thread.punch(myFighter);
	}

	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width,
			int height) {
		thread.setSurfaceSize(width, height);
	}

	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		thread.setRunning(true);
		thread.start();
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		// we have to tell thread to shut down & wait for it to finish, or else
		// it might touch the Surface after we return and explode
		boolean retry = true;
		thread.setRunning(false);
		while (retry) {
			try {
				thread.join();
				retry = false;
			} catch (InterruptedException e) {
			}
		}		
	}
	
	public void fightFinished() {
		boolean retry = true;
		thread.getAIThread().setRunning(false);
		thread.setRunning(false);
		while (retry) {
			try {
				thread.getAIThread().join();
				thread.join();
				retry = false;
			} catch (InterruptedException e) {
			}
		}
//		((Activity) context).finish();
	}

	class FightThread extends Thread {

		private boolean mRun;
		private long mLastTime;
		private SurfaceHolder mSurfaceHolder;
		public int mCanvasWidth = 1;
		public int mCanvasHeight = 1;

		private long now;
		private int millisPassed;
		private long startTime;
		private long endTime;

		public static final int FIGHTER_SIZE_X = 200;
		private final int ROTATED_FIGHTER_SIZE_X = 250;
		private final int FIGHTER_SIZE_Y = 250;
		public static final int PROXIMITY_X = 75;
		private final int WALK_SPEED = 5;
		private final int JUMP_SPEED = -10;
		private final double GRAVITY_FACTOR = 0.025;
		private final int KICK_DAMAGE = 15;
		private final int PUNCH_DAMAGE = 10;
		
		private SoundPool sounds;
//		private int mkThemeResource;
		
		private Fighter winner;
		private Fighter loser;
		
		private int rotationFactor = 1;
		
		public boolean fightFinished;
		private boolean draw;
		
		private Paint textPaint;
		private Paint lifeBarPaint;
		
		private ControlFighterThread aiThread;

		public FightThread(SurfaceHolder surfaceHolder) {
			mSurfaceHolder = surfaceHolder;
			millisPassed = 0;
			
			sounds = new SoundPool(20, AudioManager.STREAM_MUSIC, 0);
//			mkThemeResource = sounds.load(context, R.raw.mortal_kombat_theme, 1);
//			sounds.setOnLoadCompleteListener( new OnLoadCompleteListener() {
//				
//				@Override
//				public void onLoadComplete(SoundPool soundPool, int sampleId, int status) {
//					if (sampleId == mkThemeResource && status == 0)
//						sounds.play(mkThemeResource, 1.0f, 1.0f, 0, -1, 1.0f);						
//				}
//			});
			
			myFighter.loadVoiceIntegers(context, sounds);
			opponent.loadVoiceIntegers(context, sounds);
			
			textPaint = new Paint();
			textPaint.setARGB(255, 255, 118, 0);
			textPaint.setTextSize(30.0f);
			Typeface tf = Typeface.createFromAsset(context.getAssets(), "hot-mustard-btn.ttf");
			textPaint.setTypeface(tf);
			
			lifeBarPaint = new Paint();
			
			fightFinished = false;
			
			aiThread = new ControlFighterThread(this, opponent, myFighter);
			
		}

		public void run() {
			startTime = System.currentTimeMillis();
			aiThread.setRunning(true);
			aiThread.start();
//			playSound(mkThemeResource);
			while (mRun && !isInterrupted()) {
				Canvas c = null;
				try {
					c = mSurfaceHolder.lockCanvas(null);
					synchronized (mSurfaceHolder) {
						now = System.currentTimeMillis();
						millisPassed = (int) (now - mLastTime);
						if (!fightFinished && now - startTime > 60000) {
							if (myFighter.health > opponent.health)
								makeWinnerAndLoser(myFighter, opponent);
							else if (myFighter.health < opponent.health)
								makeWinnerAndLoser(opponent, myFighter);
							else
								draw();
						}
						// Log.i("Seconds passed", "" + secondsPassed);
						if (!fightFinished) {
							updatePhysics(myFighter, opponent);	
							updatePhysics(opponent, myFighter);
							doDraw(c);
						} else if (now - endTime < 10000) 
							doDrawEnding(c);
						else {
							aiThread.join();
							mRun = false;
						}
						
						mLastTime = now;
					}
				} catch (InterruptedException e) {
				} finally {
					if (c != null)
						mSurfaceHolder.unlockCanvasAndPost(c);
				}
			}
//			sounds.stop(mkThemeResource);
			Log.i("FightThread", "finished");
		}
		
		private void doDrawEnding(Canvas canvas) {
			canvas.drawBitmap(backgroundBitmap, 0, 0, null);
			if (!draw) {
				if (now - loser.currentBitmapLastUpdateTime > 300) {
					Matrix rotateMatrix = new Matrix();
					rotateMatrix.setRotate(30.0f * rotationFactor, (float) winner.x, (float) winner.y);
					loser.currentBitmap = Bitmap.createBitmap(loser.defeatBitmap, 0, 0, loser.defeatBitmap.getWidth(), loser.defeatBitmap.getHeight(), rotateMatrix, false);
					winner.currentBitmap = Bitmap.createBitmap(winner.victoryBitmap, 0, 0, winner.victoryBitmap.getWidth(), winner.victoryBitmap.getHeight(), rotateMatrix, false);
					loser.currentBitmapLastUpdateTime = now;
					winner.currentBitmapLastUpdateTime = now;
					rotationFactor = -rotationFactor;
				}
				canvas.drawBitmap(loser.currentBitmap, null, new Rect(
						(int) loser.x - (ROTATED_FIGHTER_SIZE_X / 2), (int) loser.y - (FIGHTER_SIZE_Y / 2),
						(int) loser.x + (ROTATED_FIGHTER_SIZE_X / 2), (int) loser.y + (FIGHTER_SIZE_Y / 2)), null);
				canvas.drawBitmap(winner.currentBitmap, null, new Rect(
						(int) winner.x - (ROTATED_FIGHTER_SIZE_X / 2), (int) winner.y - (FIGHTER_SIZE_Y / 2),
						(int) winner.x + (ROTATED_FIGHTER_SIZE_X / 2), (int) winner.y + (FIGHTER_SIZE_Y / 2)), null);

				textPaint.setTextAlign(Paint.Align.CENTER);
				canvas.drawText(winner.name + " is the winner!!!", mCanvasWidth / 2, mCanvasHeight / 2, textPaint);
			} else {
				if (now - myFighter.currentBitmapLastUpdateTime > 300) {
					Matrix rotateMatrix = new Matrix();
					rotateMatrix.setRotate(30.0f * rotationFactor, (float) myFighter.x, (float) myFighter.y);
					myFighter.currentBitmap = Bitmap.createBitmap(myFighter.defeatBitmap, 0, 0, myFighter.defeatBitmap.getWidth(), myFighter.defeatBitmap.getHeight(), rotateMatrix, false);
					opponent.currentBitmap = Bitmap.createBitmap(opponent.defeatBitmap, 0, 0, opponent.defeatBitmap.getWidth(), opponent.defeatBitmap.getHeight(), rotateMatrix, false);
					myFighter.currentBitmapLastUpdateTime = now;
					opponent.currentBitmapLastUpdateTime = now;
					rotationFactor = -rotationFactor;
				}
				canvas.drawBitmap(myFighter.currentBitmap, null, new Rect(
						(int) myFighter.x - (ROTATED_FIGHTER_SIZE_X / 2), (int) myFighter.y - (FIGHTER_SIZE_Y / 2),
						(int) myFighter.x + (ROTATED_FIGHTER_SIZE_X / 2), (int) myFighter.y + (FIGHTER_SIZE_Y / 2)), null);
				canvas.drawBitmap(opponent.currentBitmap, null, new Rect(
						(int) opponent.x - (ROTATED_FIGHTER_SIZE_X / 2), (int) opponent.y - (FIGHTER_SIZE_Y / 2),
						(int) opponent.x + (ROTATED_FIGHTER_SIZE_X / 2), (int) opponent.y + (FIGHTER_SIZE_Y / 2)), null);

				textPaint.setTextAlign(Paint.Align.CENTER);
				canvas.drawText("Nobody won..........", mCanvasWidth / 2, mCanvasHeight / 2, textPaint);
			}
			drawLifeBar(canvas, myFighter, 20, 10);
			drawLifeBar(canvas, opponent, mCanvasWidth - 350, 10);

			textPaint.setTextAlign(Paint.Align.LEFT);
			canvas.drawText(myFighter.name, 30, 90, textPaint);

			textPaint.setTextAlign(Paint.Align.RIGHT);
			canvas.drawText(opponent.name, mCanvasWidth - 30, 90, textPaint);

			textPaint.setTextAlign(Paint.Align.CENTER);
			textPaint.setTextSize(50);
			canvas.drawText(Integer.toString(60 - (int) ((endTime - startTime) / 1000)), mCanvasWidth / 2, 50, textPaint);
			textPaint.setTextSize(30);
						
			canvas.restore();
		}
		
		private void makeWinnerAndLoser(Fighter winner, Fighter loser) {
			myFighter.x = MY_FIGHTER_STARTING_X;
			myFighter.y = Y_REFERENCE;
			myFighter.dx = 0;
			myFighter.dy = 0;
			opponent.x = OPPONENT_STARTING_X;
			opponent.y = Y_REFERENCE;
			opponent.dx = 0;
			opponent.dy = 0;
			
			this.winner = winner;
			this.winner.currentBitmap = this.winner.victoryBitmap;
			this.loser = loser;
			this.loser.currentBitmap = this.loser.defeatBitmap;
			
			fightFinished = true;
			draw = false;
			endTime = now;
			aiThread.setRunning(false);
			
			playSound(this.loser.defeatVoiceResource);
			try {
				Thread.sleep(4000);
			} catch (InterruptedException e) {	}
			playSound(this.winner.winningVoiceResource);
		}
		
		private void draw() {
			myFighter.x = MY_FIGHTER_STARTING_X;
			myFighter.y = Y_REFERENCE;
			myFighter.dx = 0;
			myFighter.dy = 0;
			opponent.x = OPPONENT_STARTING_X;
			opponent.y = Y_REFERENCE;
			opponent.dx = 0;
			opponent.dy = 0;
			
			fightFinished = true;
			draw = true;
			endTime = now;
			aiThread.setRunning(false);

			playSound(opponent.defeatVoiceResource);
			try {
				Thread.sleep(4000);
			} catch (InterruptedException e) {	}
			playSound(myFighter.defeatVoiceResource);
		}

		private void updatePhysics(Fighter fighter, Fighter otherfighter) {			
			if ((fighter.x <= otherfighter.x && fighter.x + fighter.dx > otherfighter.x) ||
					(fighter.x >= otherfighter.x && fighter.x + fighter.dx < otherfighter.x)) {
				Log.i("updatePhysics", "mirroring bitmaps");
				fighter.mirrorBitmaps();
				otherfighter.mirrorBitmaps();
			}
			
			fighter.x += fighter.dx;
			if (fighter.x - (FIGHTER_SIZE_X / 2) < 0)
				fighter.x = (FIGHTER_SIZE_X / 2);
			else if (fighter.x + (FIGHTER_SIZE_X / 2) > mCanvasWidth)
				fighter.x = mCanvasWidth - (FIGHTER_SIZE_X / 2);
			
			fighter.y += fighter.dy;

			if (fighter.y < Y_REFERENCE) {
				fighter.dy += GRAVITY_FACTOR * millisPassed;
			} else {
				fighter.y = Y_REFERENCE;
				fighter.dy = 0;
			}

			if (now - fighter.currentBitmapLastUpdateTime > 75
					&& fighter.dx != 0) {
//				Log.i("Walk", "change of walking bitmap");
				if (fighter.currentBitmap == fighter.standBitmap)
					fighter.currentBitmap = fighter.walkBitmap;
				else if (fighter.currentBitmap == fighter.walkBitmap)
					fighter.currentBitmap = fighter.standBitmap;

				fighter.currentBitmapLastUpdateTime = now;
			}
			if (now - fighter.currentBitmapLastUpdateTime > 300
					&& (fighter.currentBitmap == fighter.kickBitmap || 
					fighter.currentBitmap == fighter.punchBitmap ||
					fighter.currentBitmap == fighter.takeHitBitmap)) {
//				Log.i("Walk", "change of hitting bitmap");
				fighter.currentBitmap = fighter.standBitmap;

				fighter.currentBitmapLastUpdateTime = now;
			}
		}

		private void doDraw(Canvas canvas) {
			canvas.drawBitmap(backgroundBitmap, 0, 0, null);
			canvas.drawBitmap(opponent.currentBitmap, null, new Rect(
					(int) opponent.x - (FIGHTER_SIZE_X / 2), (int) opponent.y - (FIGHTER_SIZE_Y / 2),
					(int) opponent.x + (FIGHTER_SIZE_X / 2), (int) opponent.y + (FIGHTER_SIZE_Y / 2)), null);
			canvas.drawBitmap(myFighter.currentBitmap, null, new Rect(
					(int) myFighter.x - (FIGHTER_SIZE_X / 2), (int) myFighter.y - (FIGHTER_SIZE_Y / 2),
					(int) myFighter.x + (FIGHTER_SIZE_X / 2), (int) myFighter.y + (FIGHTER_SIZE_Y / 2)), null);
			drawLifeBar(canvas, myFighter, 20, 10);
			drawLifeBar(canvas, opponent, mCanvasWidth - 350, 10);

			textPaint.setTextAlign(Paint.Align.LEFT);
			canvas.drawText(myFighter.name, 30, 90, textPaint);
			textPaint.setTextAlign(Paint.Align.RIGHT);
			canvas.drawText(opponent.name, mCanvasWidth - 30, 90, textPaint);
			
			textPaint.setTextAlign(Paint.Align.CENTER);
			textPaint.setTextSize(50);
			canvas.drawText(Integer.toString(60 - (int) ((now - startTime) / 1000)), mCanvasWidth / 2, 50, textPaint);
			textPaint.setTextSize(30);
			
			canvas.restore();
		}
		
		private void drawLifeBar (Canvas canvas, Fighter fighter, int left, int top) {
			double healthRatio = (double) fighter.health / Fighter.MAX_LIFE;
			if (healthRatio > 0.75)
				lifeBarPaint.setColor(Color.GREEN);
			else if (healthRatio > 0.25)
				lifeBarPaint.setARGB(255, 255, 118, 0);
			else
				lifeBarPaint.setColor(Color.RED);

			lifeBarPaint.setStyle(Paint.Style.FILL);
			RectF rectF = new RectF(new Rect(left, top, left + (int) (330 * healthRatio), top + 50));
			canvas.drawRoundRect(rectF, 10, 10, lifeBarPaint);
			
//			paint.setColor(Color.BLACK);
			lifeBarPaint.setARGB(127, 0, 0, 0	);
			lifeBarPaint.setStyle(Paint.Style.STROKE);
			lifeBarPaint.setStrokeWidth(3);
			rectF = new RectF(new Rect(left, top, left + 330, top + 50));
			canvas.drawRoundRect(rectF, 10, 10, lifeBarPaint);
		}

		public void setRunning(boolean b) {
			mRun = b;
		}

		public void setSurfaceSize(int width, int height) {
			synchronized (mSurfaceHolder) {
				mCanvasWidth = width;
				mCanvasHeight = height;
				
				Log.i("Canvas size", "Width: " + mCanvasWidth + "Height: " + mCanvasHeight);

				// resize the background image
				backgroundBitmap = Bitmap.createScaledBitmap(
						backgroundBitmap, width, height, true);
			}
		}

		public void jump(Fighter f) {
			if (!fightFinished) {
				if (f.dy == 0)
					f.dy = JUMP_SPEED;
			}

		}

		public void crouch(Fighter f) {
			if (!fightFinished && f.dy == 0) {
				f.dx = 0;
				f.currentBitmap = f.crouchBitmap;
				f.currentBitmapLastUpdateTime = now;
			}
		}

		public void stopCrouch(Fighter f) {
			if (!fightFinished) {
				f.currentBitmap = f.standBitmap;
				f.currentBitmapLastUpdateTime = now;
			}
		}

		public void walk(Fighter f, int direction) {
			if (!fightFinished) {
				if (direction > 0)
					f.dx = WALK_SPEED;
				else if (direction < 0)
					f.dx = -WALK_SPEED;
			}
		}

		public void stopWalk(Fighter f) {
			if (!fightFinished) {
				f.dx = 0;
				f.currentBitmap = f.standBitmap;
				f.currentBitmapLastUpdateTime = now;
			}
		}

		public void kick(Fighter fighter) {
			if (!fightFinished) {
				fighter.currentBitmap = fighter.kickBitmap;
				fighter.currentBitmapLastUpdateTime = now;
				hit(fighter, KICK_DAMAGE);

//				Fighter otherFighter;
//				if (fighter == myFighter)
//					otherFighter = opponent;
//				else
//					otherFighter = myFighter;
//
//				if (otherFighter.health > Fighter.MAX_LIFE / 2 && 
//						otherFighter.health - KICK_DAMAGE < Fighter.MAX_LIFE / 2)
//					playSound(fighter.winningVoiceResource);
//				else 
//					playSound(fighter.hitVoiceResource);
//				
//				if (Math.abs(fighter.x - otherFighter.x) <= PROXIMITY_X && otherFighter.y != Y_REFERENCE) {
//					takeHit(fighter, otherFighter, KICK_DAMAGE);
//				}
				
				
			}
		}

		public void punch(Fighter fighter) {
			if (!fightFinished) {
				fighter.currentBitmap = fighter.punchBitmap;
				fighter.currentBitmapLastUpdateTime = now;
				hit(fighter, PUNCH_DAMAGE);

//				Fighter otherFighter;
//				if (fighter == myFighter)
//					otherFighter = opponent;
//				else
//					otherFighter = myFighter;
//
//				if (otherFighter.health > Fighter.MAX_LIFE / 2 && 
//						otherFighter.health - PUNCH_DAMAGE < Fighter.MAX_LIFE / 2)
//					playSound(fighter.winningVoiceResource);
//				else 
//					playSound(fighter.hitVoiceResource);
//				
//				if (Math.abs(fighter.x - otherFighter.x) <= PROXIMITY_X && otherFighter.currentBitmap != otherFighter.crouchBitmap) 
//					takeHit(fighter, otherFighter, PUNCH_DAMAGE);
			}
		}

		private void hit(Fighter fighter, int damage) {
			Fighter otherFighter;
			if (fighter == myFighter)
				otherFighter = opponent;
			else
				otherFighter = myFighter;

			if (otherFighter.health > Fighter.MAX_LIFE / 2 && 
					otherFighter.health - damage < Fighter.MAX_LIFE / 2)
				playSound(fighter.winningVoiceResource);
			else 
				playSound(fighter.hitVoiceResource);
			
			if (Math.abs(fighter.x - otherFighter.x) <= PROXIMITY_X && otherFighter.currentBitmap != otherFighter.crouchBitmap) 
				takeHit(fighter, otherFighter, damage);
		}

		private void takeHit(Fighter fighterHitting, Fighter fighterReceivingHit, int damage) {
			playSound(fighterReceivingHit.takeHitVoiceResource);
			fighterReceivingHit.currentBitmap = fighterReceivingHit.takeHitBitmap;
			fighterReceivingHit.currentBitmapLastUpdateTime = now;
			fighterReceivingHit.health -= damage;
			if (fighterReceivingHit.health <= 0)
				makeWinnerAndLoser(fighterHitting, fighterReceivingHit);
		}

		private void playSound(int resource) {
			sounds.play(resource, 1.0f, 1.0f, 0, 0, 1.0f);
		}
		
		public ControlFighterThread getAIThread() {
			return aiThread;
		}
	}

}
