package uk.ac.gla.photofight.fight;

import java.util.ArrayList;
import java.util.List;

import uk.ac.gla.photofight.Fighter;
import uk.ac.gla.photofight.FighterArrayAdapter;
import uk.ac.gla.photofight.R;
import uk.ac.gla.photofight.Storage;
import uk.ac.gla.photofight.R.drawable;
import uk.ac.gla.photofight.R.id;
import uk.ac.gla.photofight.R.layout;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Typeface;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

public class SelectFightersActivity extends Activity implements
		OnItemClickListener, OnClickListener {

	private ListView myFightersListView;
	private ListView opponentsListView;
	private List<Fighter> fighterList;
	private FighterArrayAdapter fighterArrayAdapter;

	private String myFighterName;
	private String opponentName;

	private TextView myFighterNameTextView;
	private ImageView myFighterImageView;
	private TextView opponentNameTextView;
	private ImageView opponentImageView;

	private TextView fightButton;
	private TextView backButton;

	@Override
	public void onResume() {
		super.onResume();
		checkFightButton();
	}

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.select_fighters);

		myFightersListView = (ListView) findViewById(R.id.select_fighters_my_fighters_listView);
		opponentsListView = (ListView) findViewById(R.id.select_fighters_opponents_listView);

		fighterList = new ArrayList<Fighter>();
		for (String fighterName : Storage
				.getFighterNames(SelectFightersActivity.this)) {
			Fighter fighter = new Fighter(SelectFightersActivity.this,
					fighterName);
			fighterList.add(fighter);
		}

		fighterArrayAdapter = new FighterArrayAdapter(
				SelectFightersActivity.this, R.layout.fighter_row, fighterList);

		myFightersListView.setAdapter(fighterArrayAdapter);
		myFightersListView.setOnItemClickListener(this);
		opponentsListView.setAdapter(fighterArrayAdapter);
		opponentsListView.setOnItemClickListener(this);

		BitmapFactory.Options options = new BitmapFactory.Options();
		options.inSampleSize = 4;
		Bitmap defaultBitmap = BitmapFactory.decodeResource(getResources(),
				R.drawable.bgvictory_defeat, options);

		myFighterNameTextView = (TextView) findViewById(R.id.select_fighters_my_fighter_name);
		myFighterImageView = (ImageView) findViewById(R.id.select_fighters_my_fighter_imageView);
		myFighterImageView.setImageBitmap(defaultBitmap);
		opponentNameTextView = (TextView) findViewById(R.id.select_fighters_opponent_name);
		opponentImageView = (ImageView) findViewById(R.id.select_fighters_opponent_imageView);
		opponentImageView.setImageBitmap(defaultBitmap);

		fightButton = (TextView) findViewById(R.id.select_fighters_fight_button);
		fightButton.setOnClickListener(this);
		backButton = (TextView) findViewById(R.id.select_fighters_back_button);
		backButton.setOnClickListener(this);

		TextView info1TextView = (TextView) findViewById(R.id.select_fighters_info1);
		TextView info2TextView = (TextView) findViewById(R.id.select_fighters_info2);

        Typeface tf = Typeface.createFromAsset(this.getAssets(), "hot-mustard-btn.ttf");
		fightButton.setTypeface(tf);
		backButton.setTypeface(tf);
		myFighterNameTextView.setTypeface(tf);
		opponentNameTextView.setTypeface(tf);
		info1TextView.setTypeface(tf);
		info2TextView.setTypeface(tf);
		

	}

	@Override
	public void onItemClick(AdapterView<?> parent, View view, int position,
			long id) {
		Log.i("SelectFightersActivity", "onItemClick");
		if (parent.getId() == R.id.select_fighters_my_fighters_listView) {
			Log.i("SelectFightersActivity onItemClick",
					"myFightersListView clicked");
			myFighterName = fighterList.get(position).name;
			myFighterNameTextView.setText(fighterList.get(position).name);
			myFighterImageView
					.setImageBitmap(fighterList.get(position).victoryBitmap);
		} else if (parent.getId() == R.id.select_fighters_opponents_listView) {
			Log.i("SelectFightersActivity onItemClick",
					"opponentsListView clicked");
			opponentName = fighterList.get(position).name;
			opponentNameTextView.setText(fighterList.get(position).name);
			opponentImageView
					.setImageBitmap(fighterList.get(position).victoryBitmap);
		}
		checkFightButton();
	}

	private void checkFightButton() {
		if (myFighterName != null && opponentName != null)
			fightButton.setEnabled(true);
		else
			fightButton.setEnabled(false);
	}

	@Override
	public void onClick(View v) {
		if (v == fightButton) {
			SharedPreferences settings = getSharedPreferences("photoFight", 0);
			SharedPreferences.Editor editor = settings.edit();
			editor.putString("myFighterName", myFighterName);
			editor.putString("opponentName", opponentName);
			editor.commit();
			
			startActivity(new Intent(SelectFightersActivity.this,
					FightActivity.class));
			finish();
		} else if (v == backButton)
			finish();
	}

}
