package uk.ac.gla.photofight.personalize;

import java.io.File;

import uk.ac.gla.photofight.R;
import uk.ac.gla.photofight.Storage;
import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Typeface;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

public class InsertMediaToCharacterActivity extends Activity implements
		OnClickListener {

	private String fighterName;
	private ImageButton walkImageButton;
	private ImageButton standImageButton;
	private ImageButton punchImageButton;
	private ImageButton kickImageButton;
	private ImageButton takeHitImageButton;
	private ImageButton crouchImageButton;
	private ImageButton victoryImageButton;
	private ImageButton defeatImageButton;
	private ImageButton hitVoiceImageButton;
	private ImageButton takeHitVoiceImageButton;
	private ImageButton winningVoiceImageButton;
	private ImageButton victoryVoiceImageButton;
	private ImageButton defeatVoiceImageButton;

	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);

		Log.i("InsertMediaToCharacterActivity", "onActivityResult");
		
		refresh();
	}

	// refresh image button backgrounds
	private void refresh() {
		refreshButtonBackground(walkImageButton, Storage.getOutputMediaFile(InsertMediaToCharacterActivity.this, fighterName, "walk.png"));
		refreshButtonBackground(standImageButton, Storage.getOutputMediaFile(InsertMediaToCharacterActivity.this, fighterName, "stand.png"));
		refreshButtonBackground(punchImageButton, Storage.getOutputMediaFile(InsertMediaToCharacterActivity.this, fighterName, "punch.png"));
		refreshButtonBackground(kickImageButton, Storage.getOutputMediaFile(InsertMediaToCharacterActivity.this, fighterName, "kick.png"));
		refreshButtonBackground(takeHitImageButton, Storage.getOutputMediaFile(InsertMediaToCharacterActivity.this, fighterName, "take_hit.png"));
		refreshButtonBackground(crouchImageButton, Storage.getOutputMediaFile(InsertMediaToCharacterActivity.this, fighterName, "crouch.png"));
		refreshButtonBackground(victoryImageButton, Storage.getOutputMediaFile(InsertMediaToCharacterActivity.this, fighterName, "victory.png"));
		refreshButtonBackground(defeatImageButton, Storage.getOutputMediaFile(InsertMediaToCharacterActivity.this, fighterName, "defeat.png"));

		refreshButtonBackground(hitVoiceImageButton, Storage.getOutputMediaFile(InsertMediaToCharacterActivity.this, fighterName, "hit.3gpp"));
		refreshButtonBackground(takeHitVoiceImageButton, Storage.getOutputMediaFile(InsertMediaToCharacterActivity.this, fighterName, "take_hit.3gpp"));
		refreshButtonBackground(winningVoiceImageButton, Storage.getOutputMediaFile(InsertMediaToCharacterActivity.this, fighterName, "winning.3gpp"));
		refreshButtonBackground(victoryVoiceImageButton, Storage.getOutputMediaFile(InsertMediaToCharacterActivity.this, fighterName, "victory.3gpp"));
		refreshButtonBackground(defeatVoiceImageButton, Storage.getOutputMediaFile(InsertMediaToCharacterActivity.this, fighterName, "defeat.3gpp"));
	}
	
	private void refreshButtonBackground(ImageButton ib, File associatedFile) {
		if (associatedFile.exists()) {
			if (associatedFile.getName().endsWith(".png")) {
				Drawable d = Drawable.createFromPath(associatedFile.getAbsolutePath());
				ib.setBackgroundDrawable(d);
			} else if (associatedFile.getName().endsWith(".3gpp")) 
				ib.setBackgroundResource(R.drawable.microphone_blue);
		}
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.insert_media);

		SharedPreferences settings = getSharedPreferences("photoFight", 0);
		fighterName = settings.getString("fighterName", null);

		walkImageButton = (ImageButton) findViewById(R.id.insert_media_walk_imageButton);
		walkImageButton.setOnClickListener(this);

		standImageButton = (ImageButton) findViewById(R.id.insert_media_stand_imageButton);
		standImageButton.setOnClickListener(this);

		punchImageButton = (ImageButton) findViewById(R.id.insert_media_punch_imageButton);
		punchImageButton.setOnClickListener(this);

		kickImageButton = (ImageButton) findViewById(R.id.insert_media_kick_imageButton);
		kickImageButton.setOnClickListener(this);

		takeHitImageButton = (ImageButton) findViewById(R.id.insert_media_take_hit_imageButton);
		takeHitImageButton.setOnClickListener(this);

		takeHitImageButton = (ImageButton) findViewById(R.id.insert_media_take_hit_imageButton);
		takeHitImageButton.setOnClickListener(this);

		crouchImageButton = (ImageButton) findViewById(R.id.insert_media_crouch_imageButton);
		crouchImageButton.setOnClickListener(this);

		victoryImageButton = (ImageButton) findViewById(R.id.insert_media_victory_imageButton);
		victoryImageButton.setOnClickListener(this);

		defeatImageButton = (ImageButton) findViewById(R.id.insert_media_defeat_imageButton);
		defeatImageButton.setOnClickListener(this);

		hitVoiceImageButton = (ImageButton) findViewById(R.id.insert_media_hit_voice_imageButton);
		hitVoiceImageButton.setOnClickListener(this);

		takeHitVoiceImageButton = (ImageButton) findViewById(R.id.insert_media_take_hit_voice_imageButton);
		takeHitVoiceImageButton.setOnClickListener(this);

		winningVoiceImageButton = (ImageButton) findViewById(R.id.insert_media_winning_voice_imageButton);
		winningVoiceImageButton.setOnClickListener(this);

		victoryVoiceImageButton = (ImageButton) findViewById(R.id.insert_media_victory_voice_imageButton);
		victoryVoiceImageButton.setOnClickListener(this);

		defeatVoiceImageButton = (ImageButton) findViewById(R.id.insert_media_defeat_voice_imageButton);
		defeatVoiceImageButton.setOnClickListener(this);
		
		
		TextView doneButton = (TextView) findViewById(R.id.insert_media_done_button);
		doneButton.setOnClickListener(this);
		
		TextView deleteButton = (TextView) findViewById(R.id.insert_media_delete_button);
		deleteButton.setOnClickListener(this);
		
		TextView backButton = (TextView) findViewById(R.id.insert_media_back_button);
		backButton.setOnClickListener(this);

		TextView infoTextView = (TextView) findViewById(R.id.insert_media_info_textview);
		TextView info2TextView = (TextView) findViewById(R.id.insert_media_info2_textview);
		TextView info3TextView = (TextView) findViewById(R.id.insert_media_info3_textview);
		TextView charNameTextView = (TextView) findViewById(R.id.insert_media_tile_charname_textView);
		charNameTextView.setText(fighterName);

        Typeface tf = Typeface.createFromAsset(this.getAssets(), "hot-mustard-btn.ttf");
        doneButton.setTypeface(tf);
        deleteButton.setTypeface(tf);
        infoTextView.setTypeface(tf);
        info2TextView.setTypeface(tf);
        info3TextView.setTypeface(tf);
        charNameTextView.setTypeface(tf);
        backButton.setTypeface(tf);
		
		refresh();

	}

	@Override
	public void onClick(View v) {
		Intent intent;
		switch (v.getId()) {
		case R.id.insert_media_walk_imageButton:
			intent = new Intent(InsertMediaToCharacterActivity.this,
					TakePhotoActivity.class);
			intent.putExtra("type", "walk");
			startActivityForResult(intent, 0);
			break;
		case R.id.insert_media_stand_imageButton:
			intent = new Intent(InsertMediaToCharacterActivity.this,
					TakePhotoActivity.class);
			intent.putExtra("type", "stand");
			startActivityForResult(intent, 0);
			break;
		case R.id.insert_media_punch_imageButton:
			intent = new Intent(InsertMediaToCharacterActivity.this,
					TakePhotoActivity.class);
			intent.putExtra("type", "punch");
			startActivityForResult(intent, 0);
			break;
		case R.id.insert_media_kick_imageButton:
			intent = new Intent(InsertMediaToCharacterActivity.this,
					TakePhotoActivity.class);
			intent.putExtra("type", "kick");
			startActivityForResult(intent, 0);
			break;
		case R.id.insert_media_take_hit_imageButton:
			intent = new Intent(InsertMediaToCharacterActivity.this,
					TakePhotoActivity.class);
			intent.putExtra("type", "take_hit");
			startActivityForResult(intent, 0);
			break;
		case R.id.insert_media_crouch_imageButton:
			intent = new Intent(InsertMediaToCharacterActivity.this,
					TakePhotoActivity.class);
			intent.putExtra("type", "crouch");
			startActivityForResult(intent, 0);
			break;
		case R.id.insert_media_victory_imageButton:
			intent = new Intent(InsertMediaToCharacterActivity.this,
					TakePhotoActivity.class);
			intent.putExtra("type", "victory");
			startActivityForResult(intent, 0);
			break;
		case R.id.insert_media_defeat_imageButton:
			intent = new Intent(InsertMediaToCharacterActivity.this,
					TakePhotoActivity.class);
			intent.putExtra("type", "defeat");
			startActivityForResult(intent, 0);
			break;
		case R.id.insert_media_hit_voice_imageButton:
			intent = new Intent(InsertMediaToCharacterActivity.this,
					RecordVoiceActivity.class);
			intent.putExtra("type", "hit");
			startActivityForResult(intent, 0);
			break;
		case R.id.insert_media_take_hit_voice_imageButton:
			intent = new Intent(InsertMediaToCharacterActivity.this,
					RecordVoiceActivity.class);
			intent.putExtra("type", "take_hit");
			startActivityForResult(intent, 0);
			break;
		case R.id.insert_media_winning_voice_imageButton:
			intent = new Intent(InsertMediaToCharacterActivity.this,
					RecordVoiceActivity.class);
			intent.putExtra("type", "winning");
			startActivityForResult(intent, 0);
			break;
		case R.id.insert_media_victory_voice_imageButton:
			intent = new Intent(InsertMediaToCharacterActivity.this,
					RecordVoiceActivity.class);
			intent.putExtra("type", "victory");
			startActivityForResult(intent, 0);
			break;
		case R.id.insert_media_defeat_voice_imageButton:
			intent = new Intent(InsertMediaToCharacterActivity.this,
					RecordVoiceActivity.class);
			intent.putExtra("type", "defeat");
			startActivityForResult(intent, 0);
			break;
		case R.id.insert_media_done_button:
			if (Storage.isComplete(InsertMediaToCharacterActivity.this, fighterName))
				finish();
			else
				Toast.makeText(InsertMediaToCharacterActivity.this, "You have to add all photos and voices before proceding" , Toast.LENGTH_SHORT).show();
			break;
		case R.id.insert_media_delete_button:
			Storage.deleteFighter(InsertMediaToCharacterActivity.this, fighterName);
			finish();
			break;
		case R.id.insert_media_back_button:
			finish();
			break;
		}
	}

}
